@@ -7,14 +7,14 @@
    installed, parameters can be constrained as a simple
    mathematical expression of other Parameters.
 
-   version: 0.6
-   last update:  13-Aug-2012
+   version: 0.7
+   last update:  14-Sept-2012
    License: BSD
    Author:  Matthew Newville <newville@cars.uchicago.edu>
             Center for Advanced Radiation Sources,
             The University of Chicago
 """
-__version__ = '0.6'
+__version__ = '0.7'
 from .minimizer import minimize, Minimizer
 from .parameter import Parameter, Parameters
 from .confidence import conf_interval, conf_interval2d
